///////////////////////////////////////////////////////////////////////////////
// File    : view.h
// Content : view abstraction
// History : 10-Apr-2006 - [rb] creation
//           19-Oct-2007 - [rb] added CeCIIL licence, prepared distro
///////////////////////////////////////////////////////////////////////////////

/* licence ********************************************************************
Copyright Renaud Blanch [http://iihm.imag.fr/blanch/]

This software is a computer program whose purpose is to explore new interaction
techniques for zoomable treemaps.

This software is governed by the CeCILL license under French law and abiding by 
the rules of distribution of free software. You can  use, modify and/ or 
redistribute the software under the terms of the CeCILL license as circulated  
at the following URL [http://www.cecill.info]. 

As a counterpart to the access to the source code and  rights to copy, modify 
and redistribute granted by the license, users are provided only with a limited 
warranty  and the software's author,  the holder of the economic rights,  and 
the successive licensors  have only limited liability. 

The fact that you are presently reading this means that you have had knowledge 
of the CeCILL license and that you accept its terms.
******************************************************************************/

#if !defined(VIEW_H)
#define VIEW_H


/* includes ******************************************************************/

#include <vector>

#include "rectangle.h"


/* view declaration **********************************************************/

class View : Rectangle {
	// animation
	static const int STEPS_PER_LEVEL = 5;   // animation steps per level unit

	typedef std::vector< Rectangle > Animation; 
	static Animation animation;
	static int step;
	
public:
	const Rectangle &box;

protected:
	// window size (pixels)
	int window_width, window_height;
	const int MARGIN;
	const int step_min, step_max;
		
public:
	View(int W, int H, int M, int i, int a) : 
		box(*this), 
		window_width(W), window_height(H), MARGIN(M), 
		step_min(i), step_max(a) {
		dilate(MARGIN*sx(), MARGIN*sy());
	}
	
	// window size manipulation
	inline void reshape(int W, int H) { window_width = W; window_height = H; }
	
	// accessors
	inline int W() const { return window_width; }
	inline int H() const { return window_height; }
	inline double sx() const { return w()/W(); }
	inline double sy() const { return h()/H(); }
	inline double x(int X) const { return x0 + X*sx(); }
	inline double y(int Y) const { return y0 + (H()-Y)*sy(); }
	inline Rectangle target() const {
		return Rectangle(*this).dilate(-MARGIN*sx(), -MARGIN*sy()); 
	}
	
	// manipulation
	void setGLMatrix();
	
	void pan(double dx, double dy);
	void zoom(double dsx, double dsy, double x, double y);
	void zoom(double dsx, double dsy);
	void zoom(const Rectangle &r);

	static float getAnimationState() { return float(animation.size()) / step; }

protected:
	static void autozoom(int);
};


#endif // !defined(VIEW_H)
